@@ -11,33 +11,31 @@ import xarray as xr
 
 class getLidarData:
     """Windcube's data reader
-    
+
     It opens and reads the original NetCDF output
     from the Windcube lidar
-    
+
     Parameters
     ----------
 
     fileName : str
         name of the file that will be open
-    
+
     """
-    
+
     def __init__(self, fileName):
 
         self.fileName = fileName
 
-        return None
-    
     def openLidarFile(self):
 
         """
         Function to read the lidar NetCDF files
-        
+
         Returns
         -------
         tmpData : xarray.DataSet
-        
+
             a dataset from the original NetCDF files
         """
 
@@ -46,12 +44,13 @@ class getLidarData:
         tmpData.close()
 
         # retrieving the group dataset
-        tmpData = xr.open_dataset(self.fileName, group=grpName,
-                                  decode_times=False)
+        tmpData = xr.open_dataset(
+            self.fileName, group=grpName, decode_times=False
+        )
 
         # decoding time
         timeRef = tmpData.time_reference.values
-        tmpData.time.attrs['units'] = 'seconds since {0}'.format(timeRef)
+        tmpData.time.attrs["units"] = "seconds since {0}".format(timeRef)
         tmpData = xr.decode_cf(tmpData)
 
-        return tmpData
\ No newline at end of file
+        return tmpData
